/* Copyright (C) 2013-2018 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#include "algorithm.h"

static int
cmp_int(const void* a, const void* b)
{
  CHK(a != NULL);
  CHK(b != NULL);
  return *(const int*)a - *(const int*)b;
}

static int
cmp_dbl(const void* a, const void* b)
{
  CHK(a != NULL);
  CHK(b != NULL);
  return *(const double*)a < *(const double*) b
    ? -1 : (*(const double*)a > *(const double*)b ? 1 : 0);
}

int
main(int argc, char** argv)
{
  const int array_int[] = { 0, 2, 4, 4, 4, 6, 10, 11, 13, 15, 17 };
  const size_t nints = sizeof(array_int)/sizeof(int);
  const double array_dbl[] = {
    -1.123, 2.3, 2.3, 2.3, 4, 5.0, 5.2, 6, 9.1, 19.123
  };
  const size_t ndbls = sizeof(array_dbl)/sizeof(double);
  int* pi;
  int i;
  double* pd;
  double d;
  (void)argc, (void)argv;

  i = 3, pi = search_lower_bound(&i, array_int, nints, sizeof(int), cmp_int);
  CHK(*pi == 4);
  CHK(pi[3] == 6);
  i = 4, pi = search_lower_bound(&i, array_int, nints, sizeof(int), cmp_int);
  CHK(*pi == 4);
  CHK(pi[3] == 6);
  i = 5, pi = search_lower_bound(&i, array_int, nints, sizeof(int), cmp_int);
  CHK(*pi == 6);
  i = 13, pi = search_lower_bound(&i, array_int, nints, sizeof(int), cmp_int);
  CHK(*pi == 13);
  i = -1, pi = search_lower_bound(&i, array_int, nints, sizeof(int), cmp_int);
  CHK(*pi == 0);
  i = 19, pi = search_lower_bound(&i, array_int, nints, sizeof(int), cmp_int);
  CHK(pi == NULL);

  d = 2.1, pd = search_lower_bound(&d, array_dbl, ndbls, sizeof(double), cmp_dbl);
  CHK(*pd == 2.3);
  d = 2.3, pd = search_lower_bound(&d, array_dbl, ndbls, sizeof(double), cmp_dbl);
  CHK(*pd == 2.3);
  CHK(pd[3] == 4);
  d = -1.0, pd = search_lower_bound(&d, array_dbl, ndbls, sizeof(double), cmp_dbl);
  CHK(*pd == 2.3);
  d = 6.001, pd = search_lower_bound(&d, array_dbl, ndbls, sizeof(double), cmp_dbl);
  CHK(*pd == 9.1);
  d = 19.0, pd = search_lower_bound(&d, array_dbl, ndbls, sizeof(double), cmp_dbl);
  CHK(*pd == 19.123);
  d = 20.0, pd = search_lower_bound(&d, array_dbl, ndbls, sizeof(double), cmp_dbl);
  CHK(pd == NULL);

  i = -1, pi = search_lower_bound(&i, array_int, 1, sizeof(int), cmp_int);
  CHK(*pi ==  0);

  return 0;
}

